package com.weaver.seconddev.webservice;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.weaver.common.hrm.util.StringUtils;
import com.weaver.seconddev.entity.FormDataDuty;
import com.weaver.seconddev.util.DataOperateUtil;
import com.weaver.seconddev.util.InterInfoAchieveUtil;
import com.weaver.verupgrade.workflow.workflow.WorkflowAllComInfo;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

@Service("doCreateWorkflowService")
public class DoCreateWorkflowService {
    private final static Logger log = LoggerFactory.getLogger(DoCreateWorkflowService.class);

    @Autowired
    private DataOperateUtil dataOperateUtil;

    @Autowired
    private InterInfoAchieveUtil interInfoAchieveUtil;

    public String doCreateWorkflowRequest(String creator,String tenantKey,String workflowId,String mainTable,String[] detailTables) throws IOException {
        log.info("DoCreateWorkflowService start");
        List<Long> detailFormId = dataOperateUtil.getDetailFormId(workflowId);
        JSONObject param = new JSONObject();
        param.put("userid", Long.valueOf(creator));
        param.put("workflowId", Long.valueOf(workflowId));
        log.info("主表数据:" + mainTable);
        //主表数据 json {"sqr":"sqr1","date":"date1","hrm:zrr":"zrr1"}
        //字段名前面加 hrm:表示是人力资源字段
        if (mainTable == null || mainTable.equals("")) {
            return "主表数据不能为空";
        }

        JSONObject jsonObject = JSONObject.parseObject(mainTable);
        Map<String, Object> mainMap = jsonObject.toJavaObject(Map.class);
        log.error("主表集合：" + mainMap);
        List<FormDataDuty> formDataDutyList = new ArrayList();
        // 输出 Map 的内容
        for (Map.Entry<String, Object> entry : mainMap.entrySet()) {
            FormDataDuty formDataDuty = FormDataDuty.builder().build();
            String key = entry.getKey();
            Object value = entry.getValue();
            formDataDuty.setFieldType(1);
            formDataDuty.setDataKey(key);
            formDataDuty.setContent(String.valueOf(value));
            formDataDutyList.add(formDataDuty);
        }
        if(detailTables==null){

        }else{
//            log.error("处理明细数据");
            if(detailTables.length>0){
                // 遍历外部 JSONArray
                for (int i = 0; i < detailTables.length; i++) {
                    JSONArray innerArray = JSONArray.parseArray(detailTables[i]);  // 获取内部的 JSONArray
                    log.error("innerArray:"+innerArray);

                    if(innerArray.size()>0){
                        //判断明细表id是否存在
                        if (detailFormId.size() < i+1) {
                            log.error("明细表"+(i+1)+"form_id未找到或不存在");
                            return "明细表"+(i+1)+"form_id未找到或不存在";
                        }
                        Long subFormId = detailFormId.get(i);
                        log.error("明细表"+(i+1)+"form_id"+subFormId);

                        // 遍历内部 JSONArray
                        for (int j = 0; j < innerArray.size(); j++) {
                            com.alibaba.fastjson.JSONObject jsonDetail = innerArray.getJSONObject(j);

                            // 遍历 JSONObject 的键值对
                            for (String key : jsonDetail.keySet()) {
                                FormDataDuty formDataDuty = FormDataDuty.builder().build();
                                String value = jsonDetail.getString(key);
                                if (!"".equals(key)) {
//                                log.error("明细表普通字段值："+value);
                                    if(StringUtils.isNotEmpty(value)){
                                        formDataDuty.setDataKey(key);
                                        formDataDuty.setDataIndex((long) (j + 1));
                                        formDataDuty.setContent(value);
                                        //目前写死 后续在找方法
                                        formDataDuty.setSubFormId(subFormId);
                                    }else{
//                                    log.error("明细表普通字段值为空");
                                    }
                                    formDataDutyList.add(formDataDuty);
                                }

                            }

                        }
                    }

                }

                log.error("formDataDutyList数据："+formDataDutyList.size()+"-"+formDataDutyList);
            }
        }

        if(CollectionUtils.isNotEmpty(formDataDutyList)) {
            JSONObject fromDataByDataDetails = dataOperateUtil.getFromDataByDataDetails(formDataDutyList);
            param.put("formData", fromDataByDataDetails);
        }

        String code = interInfoAchieveUtil.getCode(tenantKey);
        String token = interInfoAchieveUtil.getToken(code,tenantKey);
        String url = dataOperateUtil.null2String(dataOperateUtil.getConfig("oa_address",tenantKey)) + "/api/workflow/core/paService/v1/doCreateRequest?access_token=" + token;
        log.error("url  : " +  url);

        JSONObject otherParams =new JSONObject();
        otherParams.put("isnextflow",1);
        param.put("otherParams",otherParams);

        log.error("表单json:"+param.toJSONString());
        // 远程调用创建流程。
        String response = HttpUtil.post(url, param.toJSONString());
        log.error("自动创建流程结果："+response);
        JSONObject responseObj = JSONObject.parseObject(response);
        Object message = responseObj.get("message");
        JSONObject messageObj = JSONObject.parseObject(message.toString());
        String errcode = messageObj.get("errcode").toString();

        log.error("errcode："+errcode);
        if(StringUtils.isNotEmpty(errcode)){
            if(messageObj.get("errcode").toString().equals("0")){
                log.error("requestId："+messageObj.get("requestId").toString());
                return Optional.ofNullable(messageObj.get("requestId")).map(Object::toString).orElse("");
            }else{
                log.error("errmsg："+messageObj.get("errmsg").toString());
                return messageObj.get("errmsg").toString();
            }
        }else{
            return "调用泛微流程创建接口失败";
        }
    }
}